/***************************************************************************
                          qlabyrinth.cpp  -  description
                             -------------------
    begin                : Die 19 Jun 2001 19:28:20 MEST
    copyright            : (C) 2001 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include <qsize.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qrect.h>
#include <qlabel.h>
#include <qtooltip.h>
#include "qlabyrinth.h"
#include "filesave.xpm"
#include "fileopen.xpm"
#include "filenew.xpm"
#include "goto.xpm"
#include "stop.xpm"

QLabyrinthApp::QLabyrinthApp()
{
  setCaption(tr("QLabyrinth " VERSION));

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();
  initToolBar();
  initStatusBar();

  initDoc();
  initView();

	viewToolBar->setOn(true);
  viewStatusBar->setOn(true);
	
 	QObject::connect(doc,SIGNAL(updateView(const QRect &,bool)),view,SLOT(repaint(const QRect &,bool)));
 	QObject::connect(doc,SIGNAL(newStepCount(int)),stepcounter,SLOT(display(int)));
 	QObject::connect(view,SIGNAL(moveKeyPressed(QLabyrinthDoc::orientation)),this,SLOT(slotMoveManually(QLabyrinthDoc::orientation)));
}

QLabyrinthApp::~QLabyrinthApp()
{
}

/** initializes all QActions of the application */
void QLabyrinthApp::initActions(){

  QPixmap openIcon, saveIcon, newIcon, goIcon, stopIcon;
  newIcon = QPixmap(filenew);
  openIcon = QPixmap(fileopen);
  saveIcon = QPixmap(filesave);
	goIcon = QPixmap(goto_xpm);
	stopIcon = QPixmap(stop_xpm);

	timer = new QTimer(this,"timer");
	
  fileNew = new QAction(tr("New Labyrinth"), newIcon, tr("&New"), QAccel::stringToKey(tr("Ctrl+N")), this);
  fileNew->setStatusTip(tr("Creates a new labyrinth."));
  fileNew->setWhatsThis(tr("New Labyrinth\n\nCreates a new labyrinth"));
  connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

  fileOpen = new QAction(tr("Open Labyrinth"), openIcon, tr("&Open..."), 0, this);
  fileOpen->setStatusTip(tr("Opens an existing document"));
  fileOpen->setWhatsThis(tr("Open Labyrinth\n\nOpens an existing labyrinth"));
  connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

  fileSave = new QAction(tr("Save Labyrinth"), saveIcon, tr("&Save"), QAccel::stringToKey(tr("Ctrl+S")), this);
  fileSave->setStatusTip(tr("Saves the actual labyrinth"));
  fileSave->setWhatsThis(tr("Save File.\n\nSaves the actual document"));
  connect(fileSave, SIGNAL(activated()), this, SLOT(slotFileSave()));

  fileSaveAs = new QAction(tr("Save File As"), tr("Save &as..."), 0, this);
  fileSaveAs->setStatusTip(tr("Saves the actual document under a new filename"));
  fileSaveAs->setWhatsThis(tr("Save As\n\nSaves the actual document under a new filename"));
  connect(fileSaveAs, SIGNAL(activated()), this, SLOT(slotFileSaveAs()));

  fileClose = new QAction(tr("Close Labyrinth"), tr("&Close"), QAccel::stringToKey(tr("Ctrl+W")), this);
  fileClose->setStatusTip(tr("Closes the actual labyrinth"));
  fileClose->setWhatsThis(tr("Close File\n\nCloses the actual labyrinth"));
  connect(fileClose, SIGNAL(activated()), this, SLOT(slotFileClose()));

  filePrint = new QAction(tr("Print Labyrinth"), tr("&Print"), QAccel::stringToKey(tr("Ctrl+P")), this);
  filePrint->setStatusTip(tr("Prints out the actual labyrinth"));
  filePrint->setWhatsThis(tr("Print File\n\nPrints out the actual labyrinth"));
  connect(filePrint, SIGNAL(activated()), this, SLOT(slotFilePrint()));

  fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

  viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

  viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
  connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));

	cookieGo = new QAction(tr("Go!"),goIcon,tr("&Go"),QAccel::stringToKey(tr("Ctrl+G")), this);
	cookieGo->setStatusTip(tr("let cookie run"));
	cookieGo->setWhatsThis(tr("Go!\n\nLet cookie run."));
	connect(cookieGo, SIGNAL(activated()), this, SLOT(slotCookieGo()));
	
	cookieStop = new QAction(tr("Stop!"),stopIcon,tr("&Stop"),QAccel::stringToKey(tr("Escape_Key")), this);
	cookieStop->setStatusTip(tr("let cookie stand."));
	cookieStop->setWhatsThis(tr("Stop!\n\nLet cookie rest."));
	connect(cookieStop, SIGNAL(activated()), this , SLOT(slotCookieStop()));
		
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));
	connect(timer,SIGNAL(timeout()),this,SLOT(slotSearchPath()));
	
}

void QLabyrinthApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu=new QPopupMenu();
  fileNew->addTo(fileMenu);
  fileOpen->addTo(fileMenu);
  fileClose->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileSave->addTo(fileMenu);
  fileSaveAs->addTo(fileMenu);
  fileMenu->insertSeparator();
  filePrint->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);


  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu=new QPopupMenu();
  viewMenu->setCheckable(true);
  viewToolBar->addTo(viewMenu);
  viewStatusBar->addTo(viewMenu);
  ///////////////////////////////////////////////////////////////////
  // EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(tr("&File"), fileMenu);
  menuBar()->insertItem(tr("&View"), viewMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}

void QLabyrinthApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  fileToolbar = new QToolBar(this, "file operations");
  fileNew->addTo(fileToolbar);
  fileOpen->addTo(fileToolbar);
  fileSave->addTo(fileToolbar);
  cookieGo->addTo(fileToolbar);
  cookieStop->addTo(fileToolbar);
  fileToolbar->addSeparator();
	new QLabel("steps: ",fileToolbar);
  stepcounter = new QLCDNumber(3,fileToolbar,"stepcounter");
  QToolTip::add(stepcounter,"cookies steps");
  fileToolbar->addSeparator();
	new QLabel("speed: ",fileToolbar);
	stepdurationsetting = new QSlider(25,250,25,100,QSlider::Horizontal,fileToolbar,"stepdurationsetting");
	stepdurationsetting->setMinimumSize(40,14);
	QObject::connect(stepdurationsetting,SIGNAL(valueChanged(int)),this,SLOT(slotChangeSpeed(int)));
	QToolTip::add(stepdurationsetting,"speed of cookie");
	fileToolbar->addSeparator();
  QWhatsThis::whatsThisButton(fileToolbar);

}

void QLabyrinthApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(tr("Ready."), 2000);
}

void QLabyrinthApp::initDoc()
{
	try
		{
   		doc=new QLabyrinthDoc(QSize(10,10));
   	}
   catch(QString errormsg)
   	{
   		QMessageBox::information(this,"QLabyrinth error message",errormsg);
   		qApp->quit();
   	}
}

void QLabyrinthApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  view=new QLabyrinthView(this, doc);
  setCentralWidget(view);
}

bool QLabyrinthApp::queryExit()
{
  int exit=QMessageBox::information(this, tr("Quit..."),
                                    tr("Do your really want to quit?"),
                                    QMessageBox::Ok, QMessageBox::Cancel);

  if (exit==1)
  {

  }
  else
  {

  };

  return (exit==1);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void QLabyrinthApp::slotFileNew()
{
  statusBar()->message(tr("Creating new file..."));
  MyNewLabyDialog dlg;
  if(dlg.exec() == QDialog::Accepted)
  {
  	try
  	{
  		timer->stop();
  		view->setUpdatesEnabled(false);
  		doc->newDoc(dlg.getDimensions());
  		view->setUpdatesEnabled(true);
  		view->repaint();
  	}
  	catch(QString errormsg)
   	{
   		QMessageBox::information(this,"QLabyrinth error message",errormsg);
   		qApp->quit();
   	}
  }
  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotFileOpen()
{
  statusBar()->message(tr("Opening file..."));

  QString fileName = QFileDialog::getOpenFileName(0,0,this);
  if (!fileName.isEmpty())
  {
  	try
  	{	
  		timer->stop();
  		view->setUpdatesEnabled(false);
    	doc->load(fileName);
    	view->setUpdatesEnabled(true);
    	view->repaint();
    	setCaption(fileName);
    	QString message=tr("Loaded document: ")+fileName;
    	statusBar()->message(message, 2000);
    }
    catch(QString errormsg)
    {
			QMessageBox::information(this,"QLabyrinth error message",errormsg);
    }

  }
  else
  {
    statusBar()->message(tr("Opening aborted"), 2000);
  }
}


void QLabyrinthApp::slotFileSave()
{
  statusBar()->message(tr("Saving file..."));
	try
	{
  	doc->save();
  }
  catch(QString error)
  {
	  QMessageBox::information(this,"QLabyrinth info",error);
  }
  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotFileSaveAs()
{
  statusBar()->message(tr("Saving file under new filename..."));
  QString fn = QFileDialog::getSaveFileName(0, 0, this);
  if (!fn.isEmpty())
  {
  	try
  		{
    		doc->saveAs(fn);
    	}
    catch(QString error)
    {
			QMessageBox::information(this,"QLabyrinth info",error);
    }
  }
  else
  {
    statusBar()->message(tr("Saving aborted"), 2000);
  }

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotFileClose()
{
  statusBar()->message(tr("Closing file..."));

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotFilePrint()
{
  statusBar()->message(tr("Printing..."));
  QPrinter printer;
  if (printer.setup(this))
  {
  	timer->stop();
    QPainter painter;
    painter.begin(&printer);
		
    QPixmap pix(view->size());
    pix.grabWidget(view);
    painter.drawPixmap(0,0,pix);

    painter.end();
  };

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotFileQuit()
{
  statusBar()->message(tr("Exiting application..."));
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(doc->isModified())
  {
    if(queryExit())
    {
      qApp->quit();
    }
    else
    {

    };
  }
  else
  {
    qApp->quit();
  };

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotEditCut()
{
  statusBar()->message(tr("Cutting selection..."));

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotEditCopy()
{
  statusBar()->message(tr("Copying selection to clipboard..."));

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotEditPaste()
{
  statusBar()->message(tr("Inserting clipboard contents..."));

  statusBar()->message(tr("Ready."));
}


void QLabyrinthApp::slotViewToolBar(bool toggle)
{
  statusBar()->message(tr("Toggle toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off

  if (toggle== false)
  {
    fileToolbar->hide();
  }
  else
  {
    fileToolbar->show();
  };

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotViewStatusBar(bool toggle)
{
  statusBar()->message(tr("Toggle statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off

  if (toggle == false)
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  statusBar()->message(tr("Ready."));
}

void QLabyrinthApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                      tr("QLabyrinth\nVersion " VERSION "\n(GPL) 2001 by Ruediger Knoerig\n...for Nathy in love!") );
}

/** No descriptions */
void QLabyrinthApp::slotCookieGo()
{
	timer->start(stepdurationsetting->value());
}

/**
	* called by a timer, let cookie search its way
 	*/
void QLabyrinthApp::slotSearchPath()
{
	// Did cookie reach the goal?
	if(doc->atExit())
		{
			timer->stop();
			slotEscaped();
		}
	else
	{
		// ok, lets have a look around...
		QLabyrinthDoc::fieldstate environment[4];
		for(int i=0; i < 4; i++) environment[i] = doc->whatsAt((QLabyrinthDoc::orientation) i);
		
		// search priority
		QLabyrinthDoc::fieldstate priority[2] = {QLabyrinthDoc::exit,QLabyrinthDoc::empty};
		
		bool found = false;
		int dir;
		// check all ways according to the priority list
		for(int pri = 0; pri < 2; pri++)
		{
			// check all directions
			for(dir = 0; dir < 4; dir++)
				{
					found = (environment[dir] == priority[pri]);
					if(found) break;
				}
			if(found) break;
		}
		
		if(found) doc->move((QLabyrinthDoc::orientation) dir);
		else
		{
			// last chance - remember your last step
			if(!doc->undoLastMove())
			{
				timer->stop();
				slotHopeless();
			}
		}
		
	}	
}

/** called when cookie reaches the exit */
void QLabyrinthApp::slotEscaped()
{
	QMessageBox::information(this,"QLabyrinth info","Cookie happily escaped from the WINDOZ HELPING SYSTEM\n"
	"of the evil WILL GATEZ!");
}

/** called when there is no way out */
void QLabyrinthApp::slotHopeless()
{
	QMessageBox::information(this,"QLabyrinth info","No hope left for the poor cookie...\n"
	"Another victim of the evil WILL GATEZ!");
}

/**
	* for moving  cookie manually, disabled
	* when timer is active (automove)
	*/
	
void QLabyrinthApp::slotMoveManually(QLabyrinthDoc::orientation dir)
{
	if(!timer->isActive())
		{
			if(doc->whatsAt(dir) != QLabyrinthDoc::wall) doc->move(dir);
			if(doc->atExit()) slotEscaped();
		}
}
