/***************************************************************************
                          qlabyrinth.h  -  description
                             -------------------
    begin                : Die 19 Jun 2001 19:28:20 MEST
    copyright            : (C) 2001 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QLABYRINTH_H
#define QLABYRINTH_H

// include files for QT
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qlcdnumber.h>
#include <qslider.h>
#include <qtimer.h>

// application specific includes
#include "qlabyrinthview.h"
#include "qlabyrinthdoc.h"
#include "mynewlabydialog.h"

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class QLabyrinthView is
  * created which creates your view.
  */
class QLabyrinthApp : public QMainWindow
{
  Q_OBJECT
  
  public:
    /** construtor */
    QLabyrinthApp();
    /** destructor */
    ~QLabyrinthApp();
    /** initializes all QActions of the application */
    void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /** this creates the toolbars. Change the toobar look and add new toolbars in this
     * function */
    void initToolBar();
    /** setup the statusbar */
    void initStatusBar();
    /** setup the document*/
    void initDoc();
    /** setup the mainview*/
    void initView();

    /** overloaded for Message box on last window exit */
    bool queryExit();

  public slots:

    /** generate a new document in the actual view */
    void slotFileNew();
    /** open a document */
    void slotFileOpen();
    /** save a document */
    void slotFileSave();
    /** save a document under a different filename*/
    void slotFileSaveAs();
    /** close the actual file */
    void slotFileClose();
    /** print the actual file */
    void slotFilePrint();
    /** exits the application */
    void slotFileQuit();
    /** put the marked text/object into the clipboard and remove
     * it from the document */
    void slotEditCut();
    /** put the marked text/object into the clipboard*/
    void slotEditCopy();
    /** paste the clipboard into the document*/
    void slotEditPaste();
    /** toggle the toolbar*/
    void slotViewToolBar(bool toggle);
    /** toggle the statusbar*/
    void slotViewStatusBar(bool toggle);

    /** shows an about dlg*/
    void slotHelpAbout();
    /** make cookie run */
    void slotCookieGo();
  	/** let cookie stop */
  	void slotCookieStop() {timer->stop(); }
  	/** called when there is no way out */
  	void slotHopeless();
  	/**
  		* for moving  cookie manually, disabled
			* when timer is active (automove)
		 	*/
  	void slotMoveManually(QLabyrinthDoc::orientation dir);


  private:

    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    QLabyrinthView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    QLabyrinthDoc *doc;
  
    /** file_menu contains all items of the menubar entry "File" */
    QPopupMenu *fileMenu;
    /** view_menu contains all items of the menubar entry "View" */
    QPopupMenu *viewMenu;
    /** view_menu contains all items of the menubar entry "Help" */
    QPopupMenu *helpMenu;
    /** the main toolbar */
    QToolBar *fileToolbar;
    /** actions for the application initialized in initActions() and used to en/disable them
      * according to your needs during the program */
    QAction *fileNew;
    QAction *fileOpen;
    QAction *fileSave;
    QAction *fileSaveAs;
    QAction *fileClose;
    QAction *filePrint;
    QAction *fileQuit;

    QAction *viewToolBar;
    QAction *viewStatusBar;

    QAction *helpAboutApp;
		
    QAction *cookieGo;
    QAction *cookieStop;

    /** displays the current step count of cookie */
    QLCDNumber *stepcounter;
    /** changes the duration of the delay between two steps (e.g. the speed of cookie) */
    QSlider *stepdurationsetting;
    /** Timer who induces cookies movement */
    QTimer *timer;

protected slots: // Protected slots
  /**
  	* called by a timer, let cookie search its way
 		*/
  void slotSearchPath();
  /** necessary to change timer intervalls */
  void slotChangeSpeed(int duration) {if(timer->isActive()) timer->changeInterval(duration); }
  /** called when cookie reaches the exit */
  void slotEscaped();
};
#endif 

