/***************************************************************************
                          qlabyrinthdoc.h  -  description
                             -------------------
    begin                : Die 19 Jun 2001 19:28:20 MEST
    copyright            : (C) 2001 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef QLABYRINTHDOC_H
#define QLABYRINTHDOC_H

// include files for QT
#include <qobject.h>

// application specific includes
#include "mymutex.h"
#include <stack>

extern "C"
{
#include <stdlib.h>
}

const char idstring[4] = {'Q','L','a','b'};

class QSize;
class QPoint;
class QLabyrinthApp;
class QLabyrinthView;
class QRect;

/**
  * the Document Class
  */

class QLabyrinthDoc : public QObject
{
  Q_OBJECT

	friend class QLabyrinthApp;
	friend class QLabyrinthView;
  public:
  	/** possible states of a labyrinth cell */
  	enum fieldstate {empty,wall,entry,exit,passed,passed2,cookie};
  	/** possible facings of cookie */
  	enum orientation {up,down,left,right};
  	/**
  		* default constructor
  		* @param LabySize Size of the labyrinth.
  		*/
    QLabyrinthDoc(const QSize &LabySize);
    ~QLabyrinthDoc();
    void newDoc(const QSize &LabySize);
    bool save();
    bool saveAs(const QString &filename);
    bool load(const QString &filename);
    bool isModified() const;

    /**
    	* returns true when cookie has reached the exit
    	*/
    bool atExit() { return cookiepos == exitpos; }
		
    /**
    	* returns true when cookie is at the entry point
    	*/
    bool atStart() { return cookiepos == entrypos; }

  	/**
  		* @param dir Direction of move.
  		* moves cookie in direction dir _without_ checking for correctnes
			* (use whatsAt for checking)
 			*/
  void move(QLabyrinthDoc::orientation dir);
  /** If there are protocolled moves, the last will be undone.
@return true if succeed, false if move stack is empty */
  bool undoLastMove();

  /**
  	* check adjacent fields
  	* @param Direction.
  	* @return The fieldtype of the adjacent field in direction dir
  	* (and wall, if outside of the laby) */
  QLabyrinthDoc::fieldstate whatsAt(QLabyrinthDoc::orientation dir);
	
  /**
  	* Mutex used for synchronizing access to labyrinth data
  	*/
  MyMutex datamutex;

  signals:
  	/** emitted when the whole document has changed */
    void documentChanged();
		/** emitted when a part of the view should be updated */
		void updateView(const QRect &,bool);
		/** emitted when stepcount changes */
		void newStepCount(int);
  protected:
  	/** something to save ? */
    bool modified;
    /** dimensions of the labyrinth */
    QSize dimensions;
    /** position of cookie */
    QPoint cookiepos;
    /** pointer to cookie in the lab, to improve speed */
    fieldstate *cookieptr;
    /** position of the entry */
    QPoint entrypos;
    /** position of the exit */
		QPoint exitpos;
		/** what is the current direction cookie is facing */
		orientation act_dir;
		/** frame of cookie anim*/
		int cookiestep;
		/** steps cookie taken until now  */
		int stepcount;
		/** stack which holds the last moves of cookie */
		stack<char> moves;
		/** array describing the labyrinth */
  	fieldstate *laby;
  	/** minimale Gre einer freien Labyrinthzelle */
		int minlabycelldim;
		/** Standard-Dateiname zum Sichern */
		QString filename;
  	/** generates a labyrinth */
  	void makeLaby();

  /** Erzeugt den Labyrinthinhalt nach einer rekursiven Prozedur
		* @param begin Zeiger auf die obere linke Ecke des Sublabyrinthes
		* @param width Breite des Sublabyrinthes
		* @param height Hhe des Sublabyrinthes
 		*/
  void rekursiveLaby(fieldstate *begin, int width, int height);
  typedef struct
  {
  	/** id-string (non-terminated!) */
  	unsigned long int id;
  	/** stepcounter value */
  	int stepcounter;
  	/** cookies orientation and framestep */
  	char cookieorient,cookieframe;
  	/** cookies position in the labyrinth */
 		char cookiex,cookiey;
  	/** dimensions of the labyrinth */
  	char width,height;
  }header;
};

#endif
