/***************************************************************************
                          qlabyrinthview.cpp  -  description
                             -------------------
    begin                : Die 19 Jun 2001 19:28:20 MEST
    copyright            : (C) 2001 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qlabyrinthview.h"
#include "qlabyrinthdoc.h"
#include <qsize.h>
#include "free.xpm"
#include "brick.xpm"
#include "entry.xpm"
#include "exit.xpm"
#include "passed.xpm"
#include "passed2.xpm"
#include "cookieup1.xpm"
#include "cookiedown1.xpm"
#include "cookieleft1.xpm"
#include "cookieright1.xpm"
#include "cookieup2.xpm"
#include "cookiedown2.xpm"
#include "cookieleft2.xpm"
#include "cookieright2.xpm"


QLabyrinthView::QLabyrinthView(QWidget *parent, QLabyrinthDoc *doc) : QWidget(parent)
{
	mydoc = doc;
	int width = doc->dimensions.width() << 5; // field dimension is 32x32 pixel !
	int height = doc->dimensions.height() << 5;
	setFixedSize(width,height);
	// and we want to get the input focus
	setFocusPolicy(QWidget::StrongFocus);
	
	fields = new QPixmap[6];
	fields[0] = QPixmap(free_xpm);
	fields[1] = QPixmap(brick_xpm);
	fields[2] = QPixmap(entry_xpm);
	fields[3] = QPixmap(exit_xpm);
	fields[4] = QPixmap(passed_xpm);
	fields[5] = QPixmap(passed2_xpm);
	
	cookie = new QPixmap[8];
	cookie[0] = QPixmap(cookieup1_xpm);
	cookie[1] = QPixmap(cookiedown1_xpm);
	cookie[2] = QPixmap(cookieleft1_xpm);
	cookie[3] = QPixmap(cookieright1_xpm);
	cookie[4] = QPixmap(cookieup2_xpm);
	cookie[5] = QPixmap(cookiedown2_xpm);
	cookie[6] = QPixmap(cookieleft2_xpm);
	cookie[7] = QPixmap(cookieright2_xpm);
		
	repaint();
	
  /** connect doc with the view*/
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
}

QLabyrinthView::~QLabyrinthView()
{
	if(fields) delete[] fields;
	if(cookie) delete[] cookie;
}

void QLabyrinthView::slotDocumentChanged()
{
	int width = mydoc->dimensions.width() << 5; // field dimension is 32x32 pixel !
	int height = mydoc->dimensions.height() << 5;
	setFixedSize(width,height);	
	repaint();
}

void QLabyrinthView::paintEvent(QPaintEvent *paintevent)
{
	QLabyrinthDoc::fieldstate *ptr = mydoc->laby;
	
	// if pointer to data area is a null pointer, then its
	// better to do nothing
	if(!ptr) return;
	// lock acces to labyrinth data
	while(!mydoc->datamutex.lock());
	
	int maxw = mydoc->dimensions.width();
	
	// calculates intersection of update area and widget as new update area
	QRect urect = paintevent->rect().intersect(QRect(0,0,width(),height()));
	
	// calculate the quantisized (32x32) update area coords
	int left = urect.left() >> 5;
	int top = urect.top() >> 5;
	int right = (urect.right() | 0x1F) + 1;
	int bottom = (urect.bottom() | 0x1F) + 1;
	
	int row = top * maxw;
	int col = left;
	int firstcol = col;
	
	left <<= 5;
	top <<= 5;
	
	for(int y = top; y < bottom; y += 32)
			{
				for(int x = left; x < right; x+= 32,col++)
					{
						// Remeber Commodore Amigas Blitter chip? Guy, this was funny....
						bitBlt(this,x,y,fields + ptr[col+row],0,0,32,32,Qt::CopyROP,true);
					}
				row +=maxw;
				col = firstcol;
			}
	
	// draw Cookie, if necessary
	int xpos = mydoc->cookiepos.x() << 5;
	int ypos = mydoc->cookiepos.y() << 5;
	if((xpos <= right) && (xpos >= left) && (ypos <= bottom) && (ypos >= top))
		bitBlt(this,xpos,ypos,cookie + mydoc->act_dir + (mydoc->cookiestep * 4),0,0,32,32,Qt::CopyROP,false);
		// unlock labyrinth data access
	mydoc->datamutex.unlock();
}

void QLabyrinthView::keyPressEvent(QKeyEvent *keyevent)
{
	switch(keyevent->key())
	{
		case Key_Up :
			emit moveKeyPressed(QLabyrinthDoc::up);
			break;
		case Key_Down :
			emit moveKeyPressed(QLabyrinthDoc::down);
			break;
		case Key_Left :
			emit moveKeyPressed(QLabyrinthDoc::left);
			break;
		case Key_Right :
			emit moveKeyPressed(QLabyrinthDoc::right);
			break;
	}
}
